import fse from 'fs-extra'
import extend from 'extend'
import mkdirp from 'mkdirp'
import {Promise} from 'es6-promise'

import {
  Util,
  FileParser,
  config,
  fileUtils,
  fileAttr,
  cli,
  log,
  Page,
  Hooks
} from '../'

export function save(url, tplPath, json = null, text = '', type = '', previousSave = null) {
  var p = new Promise((resolve, reject) => {

    var tplUrl = FileParser.getFileDataFromUrl(url)
    type = type || FileParser.getType(url)
    var path = dateIso(tplUrl, type)
    if(typeof previousSave !== 'undefined' && previousSave !== null){
      path.jsonPath = FileParser.changePathEnv(fileUtils.concatPath(config.root, previousSave.jsonPath.replace(config.root, '')), type)
      path.htmlPath = FileParser.changePathEnv(fileUtils.concatPath(config.root, previousSave.htmlPath.replace(config.root, '')), type)
    }

    var tpl = tplPath.replace(config.root, '')
    var fullTpl = fileUtils.concatPath(config.root, config.templates.url, tpl) + '.' + config.files.templates.extension

    if(typeof json === 'undefined' || json === null) {
      FileParser.getJson(tplUrl.json.path)
    }

    var ext = {
      template: tpl,
      link: tplUrl.publish.link,
      type: type
    }

    let meta = config.meta.name
    json[meta] = extend(json[meta], ext)
    var date = fileAttr.get(path.jsonPath).d
    if(typeof date === 'undefined' || date === null || date === '') {
      date = new Date()
    }else {
      date = new Date(date)
    }
    Util.addMetas(tpl, json, type, {}, {}, date)

    if(typeof text === 'undefined' || text === null || text === '') {
      text = fse.readFileSync(fullTpl, 'utf8')
    }
    Util.getDataList(fileUtils.removeLast(tplUrl.publish.link), text, json)
        .then(() => {
          text = Util.removeDataList(text)

          var obj = {
            type:type,
            template:{
              path: fullTpl
            },
            html: {
              path:path.htmlPath
            },
            json: {
              content: json,
              path: path.jsonPath
            }
          }
          obj = Hooks.instance.fn.preSave(obj, config)

          var page = new Page(tplUrl.publish.path, text, obj.json.content, true)

          saveJson(obj.json.path, obj.json.content)
          saveHtml(obj.html.path, page.html)

          FileParser.copySiteAssets()

          resolve({
            json: obj.json.content,
            jsonPath: obj.json.path,
            html: page.html,
            htmlPath: path.htmlPath
          })
        })
  })

  return p
}

export function saveJson(url, json) {
  mkdirp.sync(fileUtils.removeLast(url))

  if(typeof json.abe_source !== 'undefined' && json.abe_source !== null) {
    delete json.abe_source
  }

  fse.writeJsonSync(url, json, {
    space: 2,
    encoding: 'utf-8'
  })
  log.delAndWrite('Index-saveJson', `Success result writing file : ${url}`, json)
}

export function saveHtml(url, html) {
  mkdirp.sync(fileUtils.removeLast(url))
  fse.writeFileSync(url, html)
  log.delAndWrite('Index-saveHtml', `Success result writing file : ${url}`, html)
}

export function dateIso(tplUrl, type = null) {
  var newDateISO
  var dateISO
  var validate

  var saveJsonFile = tplUrl.json.path
  var saveFile = tplUrl[type].path
  var oldDateISO = fileAttr.get(saveFile).d
  
  switch(type) {
    case 'draft':
      newDateISO = (new Date().toISOString())
      dateISO = 'd' + newDateISO
      break;
    case 'publish':
      saveJsonFile = tplUrl.publish.json
      saveFile = tplUrl.publish.path
      break;
    default:
      newDateISO = (new Date().toISOString())
      dateISO = 'd' + newDateISO
      saveJsonFile = tplUrl[type].json
      saveFile = tplUrl[type].path
      FileParser.deleteFileFromName(saveFile)
      break;
  }

  if(dateISO) {
    saveJsonFile = fileAttr.add(saveJsonFile, dateISO)
    saveFile = fileAttr.add(saveFile, dateISO)
  }

  return {
    jsonPath: saveJsonFile,
    htmlPath: saveFile
  }
}